==> copilot-24.4.13-sources.jar: com/vaadin/copilot/FlowUtil.java <==
package com.vaadin.copilot;

import com.vaadin.flow.component.Component;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.internal.ComponentTracker;
import com.vaadin.flow.server.VaadinSession;
import com.vaadin.flow.server.startup.ApplicationConfiguration;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class FlowUtil {

    /**
     * Finds all Java files that are used to create the current component tree
     * for the given ui.
     *
     * @param uiId
     *            the id for the UI
     * @return a list of unique Java files
     **/
    public static Map<ComponentTracker.Location, File> findActiveJavaFiles(
            int uiId) {
        VaadinSession session = VaadinSession.getCurrent();
        try {
            session.lock();
            UI ui = session.getUIById(uiId);
            Set<ComponentTracker.Location> allComponents = getAllComponentLocations(
                    ui);
            ApplicationConfiguration applicationConfiguration = ApplicationConfiguration
                    .get(session.getService().getContext());
            Set<String> processedClasses = new HashSet<>();
            LinkedHashMap<ComponentTracker.Location, File> locations = new LinkedHashMap<>();
            for (ComponentTracker.Location location : allComponents) {
                if (processedClasses.contains(location.className())) {
                    continue;
                }
                File javaFile = location.findJavaFile(applicationConfiguration);
                // The component locator tracks the Java class and guesses the
                // Java file, so it's possible the Java file is not in the
                // project
                if (javaFile.exists()) {
                    processedClasses.add(location.className());
                    locations.put(location, javaFile);
                }
            }
            return locations;
        } finally {
            session.unlock();
        }
    }

    private static Set<ComponentTracker.Location> getAllComponentLocations(
            UI ui) {
        List<Component> componentList = new ArrayList<>();
        addComponents(ui, componentList);

        Set<ComponentTracker.Location> locations = new HashSet<>();

        for (Component component : componentList) {
            ComponentTracker.Location create = ComponentTracker
                    .findCreate(component);
            ComponentTracker.Location attach = ComponentTracker
                    .findAttach(component);
            if (create != null && attach != null) {
                // e.g. the UI has no create location, and we are not interested
                // in its attach
                locations.add(create);
                locations.add(attach);
            }
        }
        return locations;
    }

    private static void addComponents(Component component,
            List<Component> componentList) {
        componentList.add(component);
        component.getChildren().forEach(c -> addComponents(c, componentList));
    }

}

==> copilot-24.4.20-sources.jar: com/vaadin/copilot/FlowUtil.java <==
package com.vaadin.copilot;

import com.vaadin.flow.component.Component;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.internal.ComponentTracker;
import com.vaadin.flow.server.VaadinSession;
import com.vaadin.flow.server.startup.ApplicationConfiguration;
import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class FlowUtil {

    /**
     * Finds all Java files that are used to create the current component tree for the given ui.
     *
     * @param uiId the id for the UI
     * @return a list of unique Java files
     */
    public static Map<ComponentTracker.Location, File> findActiveJavaFiles(int uiId) {
        VaadinSession session = VaadinSession.getCurrent();
        try {
            session.lock();
            UI ui = session.getUIById(uiId);
            Set<ComponentTracker.Location> allComponents = getAllComponentLocations(ui);
            ApplicationConfiguration applicationConfiguration =
                    ApplicationConfiguration.get(session.getService().getContext());
            Set<String> processedClasses = new HashSet<>();
            LinkedHashMap<ComponentTracker.Location, File> locations = new LinkedHashMap<>();
            for (ComponentTracker.Location location : allComponents) {
                if (processedClasses.contains(location.className())) {
                    continue;
                }
                File javaFile = location.findJavaFile(applicationConfiguration);
                // The component locator tracks the Java class and guesses the
                // Java file, so it's possible the Java file is not in the
                // project
                if (javaFile.exists()) {
                    processedClasses.add(location.className());
                    locations.put(location, javaFile);
                }
            }
            return locations;
        } finally {
            session.unlock();
        }
    }

    private static Set<ComponentTracker.Location> getAllComponentLocations(UI ui) {
        List<Component> componentList = new ArrayList<>();
        addComponents(ui, componentList);

        Set<ComponentTracker.Location> locations = new HashSet<>();

        for (Component component : componentList) {
            ComponentTracker.Location create = ComponentTracker.findCreate(component);
            ComponentTracker.Location attach = ComponentTracker.findAttach(component);
            if (create != null && attach != null) {
                // e.g. the UI has no create location, and we are not interested
                // in its attach
                locations.add(create);
                locations.add(attach);
            }
        }
        return locations;
    }

    private static void addComponents(Component component, List<Component> componentList) {
        componentList.add(component);
        component.getChildren().forEach(c -> addComponents(c, componentList));
    }
}
